import "core-js/modules/es.symbol";
import "core-js/modules/es.symbol.description";
import "core-js/modules/es.array.is-array";
import "core-js/modules/es.object.to-string";

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

import { inject, provide } from "vue";

/**
 * Define a context that can be provided and used in the component hierarchy. Pass in a create
 * function that will be run as part of setup(). All Vue reactive methods such as `computed`
 * may be used.
 *
 * @example
 * export const userName = defineContext(() => {
 *   const userName = ref('');
 *   const uppercaseUserName = computed(() => userName.value.toUpperCase())
 *   const setUserName = (name) => {
 *     userName.value = name;
 *   }
 *   return {
 *     userName: readonly(userName),
 *     uppercaseUserName,
 *     setUserName,
 *   }
 * });
 *
 * @param {NewContextFn<C>} newContextFn Function that creates the context. Make it return an
 * object just as any other setup() function.
 * @returns {ContextDefinition<C>} A context definition object that is used by `provideContext`
 * and `useContext`.
 */
export function defineContext(newContextFn) {
  return {
    injectionKey: Symbol(),
    newContextFn: newContextFn
  };
}
/**
 * Create a context that can be provided to all components in the application setup.
 *
 * @example
 * import { userName } from '@contexts/user';
 *
 * createApp(App)
 *   .provide(...createContext(userName))
 *   .mount('#app');
 *
 * @param {ContextDefinition<C>} contextDefinition The context definition object created by
 * `defineContext`.
 * @returns {[InjectionKey<C>, C]} The injection key and the newly created context.
 */

export function createContext(_ref) {
  var injectionKey = _ref.injectionKey,
      newContextFn = _ref.newContextFn;
  return [injectionKey, newContextFn()];
}
/**
 * Provide context from a component in the hierarchy. All descendant components can then use
 * that context.
 *
 * @example
 * import { userName } from '@contexts/user';
 *
 * export default defineComponent({
 *   setup() {
 *     provideContext(userName);
 *   },
 * });
 *
 * @param {ContextDefinition<C>} contextDefinition The context definition object created by
 * `defineContext`.
 */

export function provideContext(contextDefinition) {
  provide.apply(void 0, _toConsumableArray(createContext(contextDefinition)));
}
/**
 * Use context from an ancestor component in the hierarchy.
 *
 * @example
 * import { userName } from '@contexts/user';
 *
 * export default defineComponent({
 *   setup() {
 *     const { uppercaseUserName, setUserName } = useContext(userName);
 *     return {
 *       uppercaseUserName,
 *       setUserName,
 *     }
 *   },
 * });
 *
 * @param {ContextDefinition<C>} contextDefinition The context definition object created by
 * `defineContext`.
 */

export function useContext(_ref2) {
  var injectionKey = _ref2.injectionKey;
  var context = inject(injectionKey);

  if (!context) {
    throw Error("Could not inject context, was it provided in an ancestor component or from the application?");
  }

  return context;
}
//# sourceMappingURL=index.js.map